PARAMETER lc_main
PRIVATE   lc_olddir
* Program ......: BldApp.PRG
* Author .......: Michael P. Dean
* Versions .....: dBASE IV 1.2
* Notes ........: Compile all Programs, Reports, etc ... for an application
* Syntax .......: DO BldApp WITH <main>
* Change Summary:
*  01/03/92:  Created
*  08/28/92:  Changed program type determination method.
*             Changed compiling method for REPORTS and LABELS.  New method
*             requires dummy database (included with program).
*--------------------------------------------------------------------------*
*-- Variable Definition
*   lc_main:    Main Program Name
*   ln_current: File handle for current open program
*   la_compile: Array, holds name of programs to compile
*   ln_compcnt: Array counter, number of program in array that is being compiled
*   ln_arrcnt:  Array counter, next slot in the list
*   lc_string:  String read in from current file
*   lc_talk:    Setting of SET TALK
*   ln_start:   Starting point for the Program List comparison
*   ln_curr:    Current Number for the Program List comparison
*   ln_totline: Total lines of all programs/reports, ...
*   lc_type:    Type of file (.PRG, .FRG, ...)
*   lc_name:    Name of the File
*   lc_olddir:  Starting Directory
*   lc_path:    Path to Files (if given)
*   ln_dot:     Position of the "." in the file name
*   lc_ext:     Extension for the file
*   lc_dump:    Dump Variable
*--------------------------------------------------------------------------*

lc_talk = SET("TALK")
SET TALK off
CLEAR

*-- If no parameters passed, give help screen
IF PCOUNT() = 0
  CLEAR
  TEXT
    Syntax:  DO BldApp WITH <expC>

    <expC>:  Name of main module

    Example: DO BldApp WITH "MyProg.PRG"
  ENDTEXT
  SET TALK &lc_talk
  RETURN
ENDIF

*-- Initialize variables

*-- In order to be able to compile the most files, the type of the file to
*   compile will be determined by the extension of the file name.  The
*   valid extensions (and what they correspond to) are below.  If an
*   extension is not given on the file name (which is preferrable), the
*   system will add the appropriate extension.  Here are the extensions and
*   what type of file they apply to:
*
*      PRG = Program/Procedure File/UDF/Library File
*      LBG = Label
*      FRG = Report
*      FMT = Screen Format File
*      QBE = Query

DECLARE la_compile[1170]
ln_totline    = 1
ln_arrcnt     = 2
ln_compcnt    = 1
la_compile[1] = UPPER(lc_main)
la_compile[1] = la_compile[1] + IIF(AT(".", la_compile[1]) = 0, ".PRG", "")

*-- Define Windows
DEFINE WINDOW build FROM 7, 5 TO 9,79 NONE

*-- Define Routines
ON ERROR DO ErrProc WITH ERROR(), MESSAGE()

*-- Open up dummy database (for compiling reports and labels)
USE BldApp

*-- Start analyze stage
DO WHILE TYPE("la_compile[ln_compcnt]") # "L"

  IF FILE(la_compile[ln_compcnt])
    @  9, 1 CLEAR TO 10,79

    *-- Open Program file
    IF AT(".PRG", la_compile[ln_compcnt]) > 0
      ln_current = FOPEN(la_compile[ln_compcnt])
      ln_line    = 1
      @  3, 15 CLEAR TO 5, 79
      DO WHILE .NOT. FEOF(ln_current)
        @  3, 15 SAY "Analyzing Program: " + la_compile[ln_compcnt]
        @  5, 20 SAY "Line #: " + LTRIM(STR(ln_line))

        *-- Evaluate the program line
        lc_string = UPPER(LTRIM(FGETS(ln_current)))
        IF (.NOT. LEFT(lc_string, 1) $ "*&@") .AND.;
          (LEFT(lc_string, 4) # "NOTE") .AND. .NOT. ISBLANK(lc_string)

          lc_dump = ChkLine(lc_string)
          IF .NOT. ISBLANK(lc_dump) .AND. FILE(lc_dump)
            IF .NOT. DupEntry(lc_dump)
              la_compile[ln_arrcnt] = lc_dump
              ln_arrcnt             = ln_arrcnt + 1
            ENDIF
          ENDIF
        ENDIF
        ln_totline = ln_totline + 1
        ln_line    = ln_line + 1
      ENDDO

      *-- Close the file
      ll_close = FCLOSE(ln_current)
    ENDIF

  ELSE
    ? CHR(7)
    @  9, 1 CLEAR TO 10,79
    @  9, 15 SAY "File " + la_compile[ln_compcnt] + " Not Found!"
    @ 10, 20 SAY "Possible Internal Program File"
  ENDIF

  *-- Increment the program compile counter
  ln_compcnt = ln_compcnt + 1

ENDDO
CLEAR
@  3, 5 SAY LTRIM(STR(ln_totline)) + " Lines Analyzed"

*-- Start the compile process
@  4, 1 CLEAR TO 15, 79

ln_compcnt = 1
DO WHILE TYPE("la_compile[ln_compcnt]") = "C"
  @ 5, 5 SAY "Compiling file " + la_compile[ln_compcnt]

  *-- Determine Type of file
  lc_type = SUBSTR(la_compile[ln_compcnt], (AT(".", la_compile[ln_compcnt]) + 1))
  ln_dot  = AT(".", la_compile[ln_compcnt])
  lc_file = SUBSTR(la_compile[ln_compcnt], 1, ;
            IIF(ln_dot > 0, (ln_dot - 1),;
            la_compile[ln_compcnt]))

  *-- Check for an Object File
  lc_ext = IIF("PRG" = lc_type, ".DBO", IIF("LBG" = lc_type, ".LBO",;
     IIF("FRG" = lc_type, ".FRO", IIF("FMT" = lc_type, ".FMO", ".QBO"))))

  ACTIVATE WINDOW build
  CLEAR

  DO CASE
    CASE "PRG" = lc_type

      *-- Check for a previous version, if it exists, delete it
      lc_dump = Del_File(lc_file + ".DBO")
      SET TALK on
      COMPILE (lc_file)

    CASE "LBG" = lc_type

      *-- Check for a previous version, if it exists, delete it
      lc_dump = Del_File(lc_file + ".LBO")

      *-- Turn off ON ERROR routine so the report will compile without a
      *   database in use
      ON ERROR ?? ""
      SET TALK on
      LABEL FORM &lc_file

      *-- Turn ERROR routine back on
      ON ERROR DO ErrProc WITH ERROR(), MESSAGE()

    CASE "FRG" = lc_type

      *-- Check for a previous version, if it exists, delete it
      lc_dump = Del_File(lc_file + ".FRO")

      *-- Turn off ON ERROR routine so the report will compile without a
      *   database in use
      ON ERROR ?? ""
      SET TALK on
      REPORT FORM &lc_file

      *-- Turn ERROR routine back on
      ON ERROR DO ErrProc WITH ERROR(), MESSAGE()

    CASE "FMT" = lc_type

      *-- Check for a previous version, if it exists, delete it
      lc_dump = Del_File(lc_file + ".FMO")

      SET TALK on
      SET FORMAT TO &lc_file
      SET FORMAT TO

    CASE "QBE" = lc_type

      *-- Check for a previous version, if it exists, delete it
      lc_dump = Del_File(lc_file + ".QBO")

      SET TALK on
      SET VIEW TO &lc_file
      CLOSE DATABASES
      USE BldApp

  ENDCASE
  SET TALK off
  CLEAR
  DEACTIVATE WINDOW build

  @ 5,20 CLEAR TO 5,79
  ln_compcnt = ln_compcnt + 1
ENDDO
SET TALK &lc_talk

*-- Reset System
RELEASE WINDOW build
CLOSE ALL

RETURN

*-- EOP: BldApp.PRG


FUNCTION ChkLine
PARAMETER lc_line, ln_dostart, lc_dump, ll_label
PRIVATE lc_line
* Program ......: ChkLine (UDF)
* Author .......: Michael P. Dean
* Versions .....: dBASE IV 1.2
* Notes ........: Check the line to see if there is a compilable item in it
* Syntax .......: ?/mvar = ChkLine(<line to check>)
* Change Summary:
*  01/08/92:  Created
*  08/28/92:  Bug Fix:  Fixed problems with parsing file names in ChkLine()
*             with more than one space after the execution command
*             (i.e.: DO  myprog (notice to spaces in name)
*
*             Bug Fix:  Fixed problems parsing Format Names.
*--------------------------------------------------------------------------*
*-- Variable Definition
*   lc_line:    Line to check
*   ln_dostart: Starting position of the DO command in the line
*   ln_othstrt: Starting position of a NOTE marker or other command
*   lc_dump:    Dump Variable
*   ll_label:   Is this a label (or a report)
*   lc_return:  Return Value
*--------------------------------------------------------------------------*
lc_return = ""

DO CASE

  *-- Programs
  CASE AT("DO ", lc_line) > 0
    ln_dostart = AT("DO ", lc_line)
    ln_othstrt = IIF(AT("NOTE", lc_line) = 0,;
                 IIF(AT("&" + "&", lc_line) = 0,;
                 AT("@", lc_line), AT("&" + "&", lc_line)),;
                 AT("NOTE", lc_line))
    lc_dump    = SUBSTR(lc_line, ln_dostart)

    *-- Make sure this is not a DO CASE or a DO WHILE
    IF lc_dump # "DO CASE" .AND. lc_dump # "DO WHILE" .AND.;
      (ln_othstrt = 0 .OR. ln_othstrt > ln_dostart)
      lc_dump   = LTRIM(SUBSTR(lc_dump, AT(" ", lc_dump) + 1))
      lc_return = SUBSTR(lc_dump, 1, IIF(AT(" ", lc_dump) - 1 > 0,;
                  AT(" ", lc_dump) - 1, LEN(RTRIM(lc_dump)))) + ".PRG"
    ELSE
      lc_dump = ""
    ENDIF

  *-- Procedure and Library Files
  CASE lc_line = "SET PROC" .OR. lc_line = "SET LIBR"
    lc_line   = LTRIM(SUBSTR(lc_line, AT("TO ", lc_line) + 3))
    lc_return = SUBSTR(lc_line, 1, IIF(AT(" ", lc_line) > 0,;
                (AT(" ", lc_line) - 1), LEN(lc_line))) + ".PRG"

  *-- Labels and Reports
  CASE (lc_line = "LABE" .OR. AT("LABE", lc_line) > 0 .OR.;
    lc_line = "REPO" .OR. AT("REPO", lc_line) > 0) .AND.;
    AT("FORM", lc_line) > 0
    ll_label  = (AT("LABE", lc_line) > 0)
    lc_return = LTRIM(SUBSTR(lc_line, AT("FORM ", lc_line) + 5))
    lc_return = SUBSTR(lc_return, 1, (AT(" ", lc_return) - 1)) + IIF(ll_label,;
                ".LBG", ".FRG")

  *-- Screen Forms
  CASE AT("SET FORM", lc_line) > 0
    lc_line   = SUBSTR(lc_line, AT("SET FORM", lc_line))
    lc_line   = SUBSTR(lc_line, (AT("TO ", lc_line) + 1))
    lc_line   = LTRIM(SUBSTR(lc_line, (AT(" ", lc_line) + 1)))
    lc_return = SUBSTR(lc_line, 1, IIF(AT(" ", lc_line) > 0,;
                (AT(" ", lc_line) - 1), LEN(lc_line))) + ".FMT"

  *-- Queries
  CASE AT("SET VIEW TO", lc_line) > 0
    lc_line   = SUBSTR(lc_line, AT("SET VIEW TO", lc_line))
    lc_line   = SUBSTR(lc_line, (AT("TO ", lc_line) + 1))
    lc_line   = LTRIM(SUBSTR(lc_line, (AT(" ", lc_line) + 1)))
    lc_return = SUBSTR(lc_line, 1, IIF(AT(" ", lc_line) - 1 > 0,;
                AT(" ", lc_line) - 1, LEN(RTRIM(lc_line)))) + ".QBE"

  OTHERWISE
    lc_dump = ""

ENDCASE

RETURN (LEFT(lc_return, IIF(AT(" ", lc_return) > 0, AT(" ", lc_return) - 1,;
       LEN(lc_return))))

* EOF: ChkLine

*--------------------------------------------------------------------------*

FUNCTION DupEntry
PARAMETER lc_chkthis
PRIVATE lc_chkthis, ln_arrcntr, ll_return
* Program ......: DupChk (UDF)
* Author .......: Michael P. Dean
* Versions .....: dBASE IV 1.2
* Notes ........: Check for duplicate names in the array list
* Syntax .......: DO DupChk
* Change Summary:
*
*--------------------------------------------------------------------------*
*-- Variable Definition
*   lc_chkthis:   Program to Check
*   la_compile[]: Defined in BldApp.PRG
*   ln_arrcntr:   Array counter
*   ll_return:    Return Value
*--------------------------------------------------------------------------*

*-- Initialize Variables
ln_arrcntr = 1
ll_return  = .F.

DO WHILE TYPE("la_compile[ln_arrcntr]") # "L"
  ll_return  = (la_compile[ln_arrcntr] = lc_chkthis)
  ln_arrcntr = ln_arrcntr + 1
  IF ll_return
    EXIT
  ENDIF
ENDDO

RETURN (ll_return)

*-- EOF: DupEntry (UDF)

*--------------------------------------------------------------------------*
FUNCTION Del_File
PARAMETER lc_chkfile
PRIVATE   lc_chkfile
*-------------------------------------------------------------------------*
* Author .......: Michael P. Dean
* Versions .....: dBASE IV 1.5
* Notes ........: If the passed file exists, delete it
* Syntax .......: ?/mvar = Del_File(<expC>)
* Change Summary:
*  03/05/92:  Created
*-------------------------------------------------------------------------*
*-- Variable Definition
*   lc_chkfile:  File to Check
*-------------------------------------------------------------------------*
IF FILE(lc_chkfile)
  ERASE (lc_chkfile)
ENDIF

RETURN .T.

*-- EOF: Del_File

*--------------------------------------------------------------------------*
PROCEDURE ErrProc
PARAMETER ln_errnum, lc_errmsg
PRIVATE   ln_errnum, lc_errmsg
*-------------------------------------------------------------------------*
* Author .......: Michael P. Dean
* Versions .....: dBASE IV 1.5
* Notes ........: Error Procedure
* Syntax .......: DO ErrProc WITH <expN>, <expC>
* Change Summary:
*  05/13/92:  Created
*  08/28/92:  Cleaned up display of error message
*-------------------------------------------------------------------------*
*-- Variable Definition
*   ln_errnum:   dBASE Error Number
*   lc_errmsg:   dBASE Error Message
*   lc_window:   Current Active Window
*-------------------------------------------------------------------------*
lc_window = WINDOW()
ACTIVATE SCREEN
@ 19, 5 SAY CHR(7) + "Error: "
@ 20, 5 SAY "Error Number: " + LTRIM(STR(ln_errnum))
@ 21, 5 SAY "Error Message: " + lc_errmsg
WAIT
@ 19, 1 CLEAR TO 23,79
IF .NOT. ISBLANK(lc_window)
  ACTIVATE WINDOW &lc_window
ENDIF
RETURN

*-- EOP: ErrProc
